/*
 * Licensed to the University Corporation for Advanced Internet Development, 
 * Inc. (UCAID) under one or more contributor license agreements.  See the 
 * NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The UCAID licenses this file to You under the Apache 
 * License, Version 2.0 (the "License"); you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.shibboleth.idp.authn;

import java.security.Principal;
import java.util.Collection;
import java.util.Set;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.security.auth.Subject;

import org.opensaml.storage.StorageSerializer;

import com.google.common.base.Objects;
import com.google.common.base.Predicates;
import com.google.common.collect.Collections2;

import net.shibboleth.utilities.java.support.annotation.constraint.NonnullElements;
import net.shibboleth.utilities.java.support.annotation.constraint.NotEmpty;
import net.shibboleth.utilities.java.support.annotation.constraint.Unmodifiable;
import net.shibboleth.utilities.java.support.component.IdentifiableComponent;
import net.shibboleth.utilities.java.support.logic.Constraint;
import net.shibboleth.utilities.java.support.primitive.StringSupport;

/**
 * A descriptor for an authentication flow.
 * 
 * <p>A flow models a sequence of profile actions that performs authentication in a particular way and
 * satisfies various constraints that may apply to an authentication request. Some of these constraints are
 * directly exposed as properties of the flow, and others can be found by examining the list
 * of extended {@link Principal}s that the flow exposes.</p>
 */
public class AuthenticationFlowDescriptor implements IdentifiableComponent {

    /** The unique identifier of the authentication flow. */
    private final String flowId;

    /** Whether this flow supports passive authentication. */
    private boolean supportsPassive;

    /** Whether this flow supports forced authentication. */
    private boolean supportsForced;

    /** Maximum amount of time in milliseconds, since first usage, a flow should be considered active. */
    private long lifetime;
    
    /** Maximum amount of time in milliseconds, since last usage, a flow should be considered active. */
    private long timeout;
    
    /**
     * Supported principals, indexed by type, that the flow can produce.
     * Implemented for the moment using the Subject class for convenience to allow for class-based
     * lookup in the {@link #getSupportedPrincipals} method.
     */
    @Nonnull private Subject supportedPrincipals;
    
    /** Custom serializer for the results generated by this flow. */
    @Nullable private StorageSerializer<AuthenticationResult> resultSerializer;
    
    /** Custom subject canonicalization plugin for subjects generated by this flow. */
    @Nonnull private SubjectCanonicalizer subjectCanonicalizer;

    /**
     * Constructor.
     * 
     * @param id unique ID of this flow, can not be null or empty
     */
    public AuthenticationFlowDescriptor(@Nonnull @NotEmpty final String id) {
        flowId = Constraint.isNotNull(StringSupport.trimOrNull(id), "Workflow ID cannot be null or empty");
        supportedPrincipals = new Subject();
        subjectCanonicalizer = new SimpleSubjectCanonicalizer();
    }

    /** {@inheritDoc} */
    @Nonnull @NotEmpty public String getId() {
        return flowId;
    }

    /**
     * Gets whether this flow supports passive authentication.
     * 
     * @return whether this flow supports passive authentication
     */
    public boolean isPassiveAuthenticationSupported() {
        return supportsPassive;
    }

    /**
     * Sets whether this flow supports passive authentication.
     * 
     * @param isSupported whether this flow supports passive authentication
     */
    public void setPassiveAuthenticationSupported(boolean isSupported) {
        supportsPassive = isSupported;
    }

    /**
     * Gets whether this flow supports forced authentication.
     * 
     * @return whether this flow supports forced authentication
     */
    public boolean isForcedAuthenticationSupported() {
        return supportsForced;
    }

    /**
     * Sets whether this flow supports forced authentication.
     * 
     * @param isSupported whether this flow supports forced authentication.
     */
    public void setForcedAuthenticationSupported(boolean isSupported) {
        supportsForced = isSupported;
    }

    /**
     * Gets the maximum amount of time in milliseconds, since first usage, a flow should be considered active. A
     * value of 0 indicates that there is no upper limit on the lifetime on an active flow.
     * 
     * @return maximum amount of time in milliseconds a flow should be considered active, never less than 0
     */
    public long getLifetime() {
        return lifetime;
    }

    /**
     * Sets the maximum amount of time in milliseconds, since first usage, a flow should be considered active. A
     * value of 0 indicates that there is no upper limit on the lifetime on an active flow.
     * 
     * @param flowLifetime the lifetime for the flow, must be 0 or greater
     */
    public void setLifetime(long flowLifetime) {
        lifetime = Constraint.isGreaterThanOrEqual(0, flowLifetime, "Lifetime must be greater than or equal to 0");
    }
    
    /**
     * Gets the maximum amount of time in milliseconds, since the last usage, a flow should be considered active.
     * A value of 0 indicates that there is no inactivity timeout on an active flow.
     * 
     * @return the duration.
     */
    public long getInactivityTimeout() {
        return timeout;
    }

    /**
     * Sets the maximum amount of time in milliseconds, since the last usage, a flow should be considered active.
     * A value of 0 indicates that there is no inactivity timeout on an active flow.
     * 
     * @param inactivityTimeout the flow timeout, must be 0 or greater
     */
    public void setInactivityTimeout(long inactivityTimeout) {
        timeout = Constraint.isGreaterThanOrEqual(0, inactivityTimeout,
                "Inactivity timeout must be greater than or equal to 0");
    }
    
    /**
     * Check if a result generated by this flow is still active.
     * 
     * @param result {@link AuthenticationResult} to check
     * 
     * @return  true iff the result remains valid
     */
    public boolean isResultActive(@Nonnull final AuthenticationResult result) {
        Constraint.isNotNull(result, "AuthenticationResult cannot be null");
        Constraint.isTrue(result.getAuthenticationFlowId().equals(getId()),
                "AuthenticationResult was not produced by this flow");
        
        long now = System.currentTimeMillis();
        if (getLifetime() > 0 && result.getAuthenticationInstant() + getLifetime() <= now) {
            return false;
        } else if (getInactivityTimeout() > 0 && result.getLastActivityInstant() + getInactivityTimeout() <= now) {
            return false;
        }
        
        return true;
    }
    
    /**
     * Get an immutable set of supported non-user-specific principals that the flow may produce when it operates.
     * 
     * @param <T> type of Principal to inquire on
     * @param c type of Principal to inquire on
     * 
     * @return a set of supported principals
     */
    @Nonnull @NonnullElements @Unmodifiable public <T extends Principal> Set<T> getSupportedPrincipals(Class<T> c) {
        return supportedPrincipals.getPrincipals(c);
    }

    /**
     * Get a collection of supported non-user-specific principals that the flow may produce when it operates.
     * 
     * <p>The {@link Collection#remove()} method is not supported.</p>
     * 
     * @return a live collection of supported principals
     */
    @Nonnull @NonnullElements public Collection<Principal> getSupportedPrincipals() {
        return Collections2.filter(supportedPrincipals.getPrincipals(), Predicates.notNull());
    }
    
    /**
     * Set supported non-user-specific principals that the flow may produce when it operates.
     * 
     * @param <T> a type of principal to add, if not generic
     * @param principals supported principals to add
     */
    public <T extends Principal> void setSupportedPrincipals(@Nonnull @NonnullElements final Collection<T> principals) {
        Constraint.isNotNull(principals, "Principal list cannot be null.");
        
        supportedPrincipals.getPrincipals().clear();
        supportedPrincipals.getPrincipals().addAll(Collections2.filter(principals, Predicates.notNull()));
    }
   
    /**
     * Get the custom serializer for results produced by this flow.
     * 
     * @return the custom serializer, or null
     */
    @Nullable public StorageSerializer<AuthenticationResult> getResultSerializer() {
        return resultSerializer;
    }

    /**
     * Set the custom serializer for results produced by this flow.
     * 
     * @param serializer the custom serializer, or null
     */
    public void setResultSerializer(@Nullable final StorageSerializer<AuthenticationResult> serializer) {
        resultSerializer = serializer;
    }

    /**
     * Get the custom canonicalizer for subjects produced by this flow.
     * 
     * @return the custom canonicalizer, or null
     */
    @Nonnull public SubjectCanonicalizer getSubjectCanonicalizer() {
        return subjectCanonicalizer;
    }

    /**
     * Set the custom canonicalizer for subjects produced by this flow.
     * 
     * @param canonicalizer the custom canonicalizer, or null
     */
    public void setSubjectCanonicalizer(@Nonnull final SubjectCanonicalizer canonicalizer) {
        subjectCanonicalizer = Constraint.isNotNull(canonicalizer, "SubjectCanonicalizer cannot be null");
    }
        
    /** {@inheritDoc} */
    public int hashCode() {
        return flowId.hashCode();
    }

    /** {@inheritDoc} */
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }

        if (obj == this) {
            return true;
        }

        if (obj instanceof AuthenticationFlowDescriptor) {
            return flowId.equals(((AuthenticationFlowDescriptor) obj).getId());
        }

        return false;
    }

    /** {@inheritDoc} */
    public String toString() {
        return Objects.toStringHelper(this).add("flowId", flowId).add("supportsPassive", supportsPassive)
                .add("supportsForcedAuthentication", supportsForced).add("lifetime", lifetime)
                .add("inactivityTimeout", timeout).toString();
    }
}